package com.yizhi.wechat.application.feign;

import com.yizhi.wechat.application.vo.wechat.domain.PublicPlatformConfigVo;
import com.yizhi.wechat.application.vo.wechat.domain.TrAccountUserinfoVo;
import com.yizhi.wechat.application.vo.wechat.domain.UserInfoVo;
import com.yizhi.wechat.application.vo.wechat.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@FeignClient(name = "wechat", contextId = "WeiXinClient")
public interface WeiXinClient {
    /**
     * 查询用户与微信公众号绑定的状态
     *
     * @param wechatUserId 用户微信id
     * @return 关联关系实体
     */
    @ResponseBody
    @GetMapping(value = "/wechat/get/user")
    TrAccountUserinfoVo queryUserInWechat(
            @RequestParam(name = "wechatUserId") String wechatUserId
            , @RequestParam(name = "companyId") Long companyId
    );

    /**
     * 获取微信公众服务号用户的信息
     *
     * @param appid APPID
     * @param code  CODE
     */
    @RequestMapping(value = "/wechat/get/wechatuser", method = RequestMethod.GET)
    public WechatUserInfoVO getUserInfo(@RequestParam(value = "code") String code,
                                        @RequestParam(value = "appid") String appid,
                                        @RequestParam(value = "agentId", required = false) String agentId,
                                        @RequestParam(value = "source", required = false) Integer source
    );


    /**
     * 保存微信id和系统用户id的关系
     *
     * @param wechatUserId 用户的微信di
     * @param accountId    系统账号的id
     * @return TrAccountInfo
     */
    @RequestMapping(value = "/trAccountUserinfo/save/accountinfo", method = RequestMethod.GET)
    public TrAccountUserinfoVO saveTrAccountUserinfo(@RequestParam(value = "wechatUserId", required = true) String wechatUserId,
                                                     @RequestParam(value = "accountId", required = true) Long accountId,
                                                     @RequestParam(value = "siteId", required = true) Long siteId,
                                                     @RequestParam(value = "companyId", required = true) Long companyId,
                                                     @RequestParam(value = "terminalType", required = false) Integer terminalType);

    /**
     * 获取用户的签名
     *
     * @param agentId 机构Id
     * @param url
     * @return
     */
    @RequestMapping(value = "/wechat/get/signature", method = RequestMethod.GET)
    public SignatureVO getSignature(
            @RequestParam(name = "appid") String appid,
            @RequestParam(name = "url") String url,
            @RequestParam(name = "agentId") String agentId);

    /**
     * 获取授权token
     *
     * @param publicPlatformConfig 参数实体
     * @return
     */
    @GetMapping(value = "/wechat/get/token")
    public String getChatAccessToken(PublicPlatformConfigVo publicPlatformConfig);

    /**
     * 获取公众号的配置信息
     *
     * @param appid
     * @return
     */
    @RequestMapping(value = "/wechat/get/platformconfig", method = RequestMethod.GET)
    public PublicPlatformConfigVo getPlatformconfig(@RequestParam(value = "appid") String appid,
                                                  @RequestParam(value = "agentId") String agentId);

    /**
     * 保存token到redis
     */
    @PostMapping(value = "/wechat/save/token")
    public void saveAccessTokenToRedis();

    /**
     * 删除
     *
     * @param ids 用户id列表
     * @return
     */
    @PostMapping(value = "/trAccountUserinfo/del/relations")
    public ResponseVO delRelation(@RequestBody List<Long> ids);

    /**
     * 新增平台配置信息
     *
     * @param publicPlatformConfigVO
     * @return
     */
    @PostMapping(value = "/publicPlatformConfig/insert")
    public boolean insertPublicPlatformConfigInfo(@RequestBody PublicPlatformConfigVO publicPlatformConfigVO);

    /**
     * 通过用户的id获取微信openId
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "/trAccountUserinfo/get/wechatid")
    public String getWechatIdbyAccountId(@RequestParam(name = "accountId") Long accountId);

    /**
     * TODO 8月底撤掉
     *
     * @return
     */
    @PostMapping(value = "/wechat/update/user")
    boolean updateUserInWechat(@RequestBody TrAccountUserinfoVo trAccountUserinfo
    );

    /**
     * job 定期执行任务把
     * jsapiticket 放到redis中
     *
     * @date 2018-8-20 20:47:53
     */
    @PostMapping(value = "/wechat/save/jsapiticket")
    public void saveJsApiTicket();

    /**
     * 获取组织列表和用户信息
     *
     * @param accessToken
     * @param wechatUserId
     * @return
     */
    @GetMapping(value = "/wechat/get/getWechatUser")
    public WeChatUserVO getWechatUser(@RequestParam(name = "accessToken") String accessToken,
                                      @RequestParam(name = "wechatUserId") String wechatUserId);

    /**
     * 获取用户的关联关系
     *
     * @return
     */
    @PostMapping(value = "/trAccountUserinfo/relation/get")
    TrAccountUserinfoVo getAccountUserInfo(@RequestBody TrAccountUserinfoVo trAccountUserinfo);

    /**
     * 保存关联关系
     *
     * @param userInfoVO
     * @return
     */
    @PostMapping(value = "/trAccountUserinfo/save")
    TrAccountUserinfoVO saveWechatUserInfo(@RequestBody TrAccountUserinfoVO userInfoVO);


    /**
     * 用户停用
     *
     * @param userInfo
     * @return
     */
    @PostMapping(value = "/trAccountUserinfo/stop")
    Boolean updateTrAccountUserInfo(@RequestBody TrAccountUserinfoVo userInfo);

    @PostMapping(value = "/trAccountUserinfo/unbind/weChatUserId")
    Boolean unbindAccountByWeChatUserId(@RequestParam(name = "weChatUserId", required = true) String weChatUserId);

    /**
     * 批量解绑用户 by weChatUserId
     *
     * @param code
     * @param appId
     * @param agentId
     * @return
     */
    @PostMapping(value = "/trAccountUserinfo/unbind/batch/weChatUserId")
    Boolean unbindAccount(
            @RequestParam(value = "code") String code,
            @RequestParam(value = "appId") String appId,
            @RequestParam(value = "agentId", required = false) String agentId);

    /**
     * 批量解绑用户
     * @param accountIds 用户id
     * @return··
     */
    @PostMapping(value = "/trAccountUserinfo/unbindUser")
    Boolean unbindUser(@RequestBody List<Long> accountIds);


    @PostMapping(value = "/trAccountUserinfo/get/bind/relation")
    Map<Long,TrAccountUserinfoVo> getUserRelations(@RequestBody List<Long> accountIds);

    /**
     * 获取绑定关系
     * @param authorizeUsersVO
     * @return
     */
    @PostMapping(value = "/trAccountUserinfo/get/bind/relation")
    Map<Long,TrAccountUserinfoVo> getUserRelations(@RequestBody AuthorizeUsersVO authorizeUsersVO);

    /**
     * 获取微信用户的信息
     * @param wechatIds
     * @return
     */
    @PostMapping("wechat/get/userInfo/wechatIds")
    Map<String, UserInfoVo> getWeChatUserInfoList(@RequestBody List<String> wechatIds);
}