package com.yizhi.application.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.service.IPublicPlatformConfigService;
import com.yizhi.application.service.ITrAccountUserinfoService;
import com.yizhi.application.task.UpdateUserInfoService;
import com.yizhi.wechat.application.utils.HttpClientUtils;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.MiniProgramVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 第三方用户信息获取类
 *
 * @author lingye
 * @date 2020-1-2
 */
@RestController
@RequestMapping(value = "/miniProgram")
public class MiniProgramController {

    public static final Logger LOGGER = LoggerFactory.getLogger(MiniProgramController.class);
    HttpClientUtils httpClientUtils = new HttpClientUtils();
    @Value("${cloud.min.program.base_url}")
    private String baseUrl;
    @Autowired
    private IPublicPlatformConfigService iPublicPlatformConfigService;

    @Autowired
    private ITrAccountUserinfoService iTrAccountUserinfoService;

    @Autowired
    private UpdateUserInfoService updateUserInfoService;

    @GetMapping(value = "/get")
    public MiniProgramVo getMiniProgram(
            @RequestParam(name = "appId", required = true) String appId,
            @RequestParam(name = "code", required = true) String code,
            @RequestParam(name = "headImg", required = false) String headImg
    ) {
        RequestContext context = ContextHolder.get();
        LOGGER.info("上下文信息：{}", JSON.toJSON(context));
        // 获取配置信息
        LOGGER.info("签名接口参数appId：" + appId);
        PublicPlatformConfig publicPlatformConfig = iPublicPlatformConfigService.getPubConfig(appId, null);
        String url = baseUrl + "?appid=" + appId + "&secret=" + publicPlatformConfig.getSecret() + "&js_code=" + code + "&grant_type=authorization_code";
        String ret = httpClientUtils.httpGet(url);
        LOGGER.info("获取unionId 和 openId 的结果：{}", JSON.toJSON(ret));
        JSONObject retJson = JSONObject.parseObject(ret);
        MiniProgramVo miniProgramVo = new MiniProgramVo();
//        String errcode = retJson.getString("errcode");
//        if (!Constants.ZERO.equals(errcode)) {
//            String errMsg = retJson.getString("errmsg");
//            miniProgramVo.setCode(errcode);
//            miniProgramVo.setMsg(errMsg);
//            return miniProgramVo;
//        }


        boolean result = false;
        TrAccountUserinfo userInfo = new TrAccountUserinfo();
        String openId = null;
        String unionId = null;
        TrAccountUserinfo account = null;

        if (publicPlatformConfig.getOpenPlateFormBind() == 1) {
            unionId = retJson.getString("unionid");
            LOGGER.info("获取到的unionId:{}", unionId);
            if (unionId != null) {
                miniProgramVo.setUnionId(unionId);
                userInfo.setUnionId(unionId);
                account = findBindWeChat(userInfo, context.getCompanyId());
            } else {
                result = false ;
            }

        } else {
            openId = retJson.getString("openid");
            LOGGER.info("获取到的openId:{}", openId);

            if (openId != null) {
                miniProgramVo.setOpenId(openId);
                userInfo.setWechatuserid(openId);
                account = findBindWeChat(userInfo, context.getCompanyId());
            } else {
                result = false ;
            }
        }
        // 异步新增用户信息到用户表
        updateUserInfoService.addMiniProgramUserInfo(openId, unionId);

        LOGGER.info("返回的关系表：{}",JSON.toJSON(account));

        miniProgramVo.setCode(Constants.ZERO);

        miniProgramVo.setAccountId(account != null ? account.getAccountId() : 0L);
        if (null != account) {
            updateUserInfoService.taskUpdateMiniUserInfo(account.getAccountId(), headImg);
            result = true;
        }
        miniProgramVo.setRet(result);
        return miniProgramVo;
    }

    public TrAccountUserinfo findBindWeChat(TrAccountUserinfo userInfo,Long companyId){
        userInfo.setIsDeleted(0);
        userInfo.setCompanyId(companyId);
        return iTrAccountUserinfoService.findBindWechat(userInfo);
    }
}
