package com.yizhi.application.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.*;
import com.yizhi.wechat.application.vo.wechat.*;
import com.yizhi.wechat.application.vo.wechat.AccountVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lilingye
 * @since 2018-04-29
 */
@RestController
@RequestMapping("/wechat")
public class UserInfoController {

    private static final Logger LOGGER = LoggerFactory.getLogger(UserInfoController.class);

    @Autowired
    private IUserInfoService iUserInfoService;

    @Autowired
    private IPublicPlatformConfigService iPublicPlatformConfigService;

    @Autowired
    private ITrAccountUserinfoService iTrAccountUserinfoService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ICwechatService cwechatService;

    @Autowired
    IAccessTokenService accessTokenService;
    @Autowired
    IProviderUserInfoService providerUserInfoService;


    /**
     * 查询用户是否在微信服务中绑定
     *
     * @param wechatUserId
     * @return
     */
    @GetMapping(value = "/get/user")
    public TrAccountUserinfo queryUserInWechat(
            @RequestParam(name = "wechatUserId") String wechatUserId,
            @RequestParam(name = "companyId") Long companyId
    ) {
        // 获取id
        TrAccountUserinfo trAccountUserinfo = iTrAccountUserinfoService.findAccount(wechatUserId, companyId);
        if (trAccountUserinfo != null) {
            LOGGER.info("wechatUserId为：" + wechatUserId + ",存在数据库表中");
            return trAccountUserinfo;
        } else {
            LOGGER.info("wechatUserId为：" + wechatUserId + ",不存在数据库表中");
            return null;
        }
    }

    /**
     * 获取微信公众服务号用户的信息
     *
     * @param appid 微信公众号的appid
     * @param code  条用微信api获取的code
     */
    @RequestMapping(value = "/get/wechatuser", method = RequestMethod.GET)
    public WechatUserInfoVO getUserInfo(
            @RequestParam(value = "code") String code,
            @RequestParam(value = "appid") String appid,
            @RequestParam(value = "agentId", required = false) String agentId,
            @RequestParam(value = "source", required = false) Integer source
    ) {
        LOGGER.info("-------------------------------------------");
        LOGGER.info("企业的appid：{}", appid);
        LOGGER.info("参数code:{}", code);
        LOGGER.info("参数agentId:{}", agentId);
        LOGGER.info("请求时间：{}",JSON.toJSON(new Date()));
        LOGGER.info("--------------------------------------------");

        WechatUserInfoVO userInfoVO = new WechatUserInfoVO();
        List<PublicPlatformConfig> pubConfigs = iPublicPlatformConfigService.getPubConfigs(appid, agentId);
        LOGGER.info("公众号配置所有列表：{}", JSON.toJSON(pubConfigs));
        if (CollectionUtils.isEmpty(pubConfigs)) {
            LOGGER.info("公众号配置获取为空，请检查参数！！！");
            return userInfoVO;
        }

        PublicPlatformConfig publicPlatformConfig = pubConfigs.get(0);
        // 获取微信公众号维护的配置信息
//        PublicPlatformConfig publicPlatformConfig = iPublicPlatformConfigService.getPubConfig(appid, agentId);

            if ("0".equals(publicPlatformConfig.getType())) {
                // 企业服务号
                userInfoVO = getCUserInfo(code, appid, publicPlatformConfig.getSecret(), agentId,source);
            } else if ("1".equals(publicPlatformConfig.getType())) {
                //  微信服务号
                userInfoVO = iUserInfoService.getWeChatUserInfo(appid, publicPlatformConfig.getSecret(), code,source);
            } else if ("2".equals(publicPlatformConfig.getType())) {
                userInfoVO = providerUserInfoService.getProviderInfoDetail(appid,code);
            }
            return userInfoVO;
    }

    /**
     * 获取微信企业服务号用户的信息
     *
     * @param appid 企业的CorpID
     * @param code  调用微信api获取的code
     */
    @ResponseBody
    @RequestMapping(value = "/get/cwechatuser", method = RequestMethod.GET)
    public WechatUserInfoVO getCUserInfo(
            @RequestParam(value = "code", required = true) String code,
            @RequestParam(value = "appid", required = true) String appid,
            @RequestParam(value = "secret") String secret,
            @RequestParam(value = "agentId",required = false) String agentId,
            @RequestParam(value = "source" ,required = false) Integer source
    ) {
        LOGGER.info("企业服务号的传来参数appid" + appid);
        LOGGER.info("传来的参数code:" + code);
        LOGGER.info("传来的参数secret:" + secret);
        WUserInfoVO userInfoVO = new WUserInfoVO();
        // 组装实体 提供参数
        PublicPlatformConfig publicPlatformConfig = new PublicPlatformConfig();
        publicPlatformConfig.setAppId(appid);
        publicPlatformConfig.setAgentId(agentId);
        publicPlatformConfig.setSecret(secret);
        return iUserInfoService.getCompanyWechatUserInfo(publicPlatformConfig, code,source);
    }

    /**
     * 获取的签名
     *
     * @param url
     * @since 2018-5-23 17:45:32
     */
    @ResponseBody
    @RequestMapping(value = "/get/signature", method = RequestMethod.GET)
    public SignatureVO getSignature(
            @RequestParam(name = "appid") String appid,
            @RequestParam(name = "url") String url,
            @RequestParam(name = "agentId") String agentId
    ) {
        // 获取配置信息
        LOGGER.info("签名接口参数appid：" + appid + ";接口参数agentId:" + agentId + ";接口参数 url:" + url);
        PublicPlatformConfig publicPlatformConfig = iPublicPlatformConfigService.getPubConfig(appid, agentId);
        LOGGER.info("服务号的类型为：" + publicPlatformConfig);
        if (publicPlatformConfig != null) {
            LOGGER.info("服务号的类型为：" + publicPlatformConfig.getType());
        } else {
            LOGGER.error("配置信息查询未空,请检查传参是否正确！！！");
            return null;
        }
//        首先获取access——token
        String access_token = getChatAccessToken(publicPlatformConfig);
        LOGGER.info("access_token:" + access_token);
        return iUserInfoService.getSignature(access_token, url, publicPlatformConfig);
    }

    /**
     * 获取授权的AccessToken
     *
     * @return
     * @since 2018-5-25 14:35:32
     */
    @ResponseBody
    @GetMapping(value = "/get/token")
    public String getChatAccessToken(@RequestBody PublicPlatformConfig publicPlatformConfig) {

        String redisKey = null;
        if ("0".equals(publicPlatformConfig.getType())) {
            redisKey = (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
            LOGGER.info("类型为0 的redisKey：" + redisKey);
        } else if ("1".equals(publicPlatformConfig.getType())) {
            LOGGER.info("类型为1 的redisKey：" + redisKey);
            redisKey = publicPlatformConfig.getAppId().trim();
        } else if ("2".equals(publicPlatformConfig.getType())){
            redisKey = publicPlatformConfig.getAppId().trim();
        }
        LOGGER.info("redisKey的值:" + redisKey);
        //从缓存中获取token

        Object tokenVO = redisCache.hget(redisKey, redisKey);

        LOGGER.info("类型为0 的redisKey：" + redisKey);
        // 声明token
        String access_token = null;
        // 判断缓存中的redis存在
        if (tokenVO != null) {

            //获取token
            Object tokenJSON = JSON.toJSON(tokenVO);
            LOGGER.info("查看缓存中的access_token信息：" + tokenJSON.toString());
                LOGGER.info(JSON.parseObject(tokenJSON.toString()).getString("accessToken"));
                String accessToken = JSON.parseObject(tokenJSON.toString()).getString("accessToken");

            if (!"".equals(accessToken) && !"null".equals(accessToken) && accessToken != null) {
                LOGGER.info("accessToken值存在");
                return JSON.parseObject(tokenJSON.toString()).getString("accessToken");
            } else {
                //生成新的token
                LOGGER.info("accessToken值不在在，重新获取");
                return getAccessToken(publicPlatformConfig);
            }
        } else {
            // 生成新的token
            return getAccessToken(publicPlatformConfig);

        }
    }

    /**
     * 获取公众号的配置信息
     *
     * @param appid 公众号的appdId
     * @return
     * @since 2018-5-25 14:35:24
     */
    @RequestMapping(value = "/get/platformconfig", method = RequestMethod.GET)
    public PublicPlatformConfig getPlatformconfig(@RequestParam(value = "appid") String appid,
                                                  @RequestParam(value = "agentId") String agentId) {
        // 获取微信公众号维护的配置信息
        PublicPlatformConfig publicPlatformConfig = iPublicPlatformConfigService.getPubConfig(appid, agentId);
        if (publicPlatformConfig != null) {
            return publicPlatformConfig;
        } else {
            return null;
        }
    }

    /**
     * JOB 执行任务
     * 从微信接口中获取token和jsapiticket存放到缓存Redis中
     */
    @PostMapping(value = "/save/token")
    public void saveAccessTokenToRedis() {

        List<PublicPlatformConfig> publicPlatformConfigList = iPublicPlatformConfigService.getPlatformInfos();

        for (PublicPlatformConfig platformConfig : publicPlatformConfigList
        ) {
            try {
                // 调用微信提供的API接口获取access_token 并存在redis中
                String accessToken = accessTokenService.insetWeChatAccessToken(platformConfig);
                accessTokenService.addJsApiTicket(accessToken, platformConfig);
            } catch (Exception e) {
                LOGGER.info(e + "");
            }
        }
    }

    /**
     * 获取token
     * @param publicPlatformConfig
     * @return
     */
    @PostMapping("/get/weChatToken")
    public String getWeChatAccessToken(PublicPlatformConfig publicPlatformConfig) {
        return accessTokenService.getAccessToken(publicPlatformConfig);
    }

    /**
     * 生成新的accessToken存在redis中
     *
     * @param publicPlatformConfig 配置类
     * @return String
     * @Date 2018年7月23日16:28:54
     * @author lilingye
     */
    public String getAccessToken(PublicPlatformConfig publicPlatformConfig) {

        Date nowDate = new Date();
        WTokenVO wTokenVO = new WTokenVO();
        wTokenVO.setAppid(publicPlatformConfig.getAppId());
        wTokenVO.setCreateTime(nowDate);
        wTokenVO.setAccessToken(publicPlatformConfig.getAgentId());

        // 调用微信接口获取token 并存在redis中
        String access_token = iUserInfoService.getAccessToken(publicPlatformConfig);
        LOGGER.info("appid的值是" + publicPlatformConfig.getAppId());
        LOGGER.info("secret值是" + publicPlatformConfig.getSecret());
        LOGGER.info("type值是" + publicPlatformConfig.getType());
        LOGGER.info("生成的access_token的值：" + access_token);

        String redisKey = null;

        if ("0".equals(publicPlatformConfig.getType())) {
            redisKey = (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        } else {
            redisKey = publicPlatformConfig.getAppId();
        }

        if (!"null".equals(access_token) && null != access_token && !"".equals(access_token)) {
            wTokenVO.setAccessToken(access_token);
            redisCache.hset(redisKey, redisKey, JSON.toJSONString(wTokenVO), 3600);
            return access_token;
        } else {
            return access_token;
        }
    }

    /**
     * @return
     */
    @PostMapping(value = "/update/user")
    public boolean updateUserInWechat(@RequestBody TrAccountUserinfo trAccountUserinfo
    ) {
        // 获取id
        RequestContext context = ContextHolder.get();
        trAccountUserinfo.setCompanyId(context.getSiteId());
        LOGGER.info("=====================================================================");

        LOGGER.info("公司id" + context.getSiteId());

        LOGGER.info("=====================================================================");
        LOGGER.info("更新siteId");
        return iTrAccountUserinfoService.updateById(trAccountUserinfo);
    }


    /**
     * 获取企业用户信息
     *
     * @param accessToken
     * @param wechatUserId
     * @return 用户详细信息
     * @date 2018-8-22 10:05:41
     */
    @GetMapping(value = "/get/getWechatUser")
    public WeChatUserVO getWechatUser(@RequestParam(name = "accessToken") String accessToken,
                                      @RequestParam(name = "wechatUserId") String wechatUserId) {
        LOGGER.info("参数accessToken：" + accessToken + ";wechatUserId是：" + wechatUserId);
        WeChatUserVO weChatUserVO = new WeChatUserVO();
        // 获取企业微信的微信的信息
        WUserInfoVO wUserInfoVO = cwechatService.getCwechatUserDetail(accessToken, wechatUserId);
        Long orgId = null;
        LOGGER.info("拿到的企业部门id：" + wUserInfoVO.getDepartmentId());
        if (!"".equals(wUserInfoVO.getDepartmentId())) {
            orgId = Long.valueOf(wUserInfoVO.getDepartmentId());

            // 获取目前组织的名称列表
            List<String> orgNameList = cwechatService.getCwechatOrg(accessToken, orgId);

            weChatUserVO.setOrgNames(orgNameList);

            AccountVO accountVO = new AccountVO();
            accountVO.setName(wUserInfoVO.getWechatuserid());
            accountVO.setEmail(wUserInfoVO.getEmail());
            accountVO.setMobile(wUserInfoVO.getMobile());
            accountVO.setPosition(wUserInfoVO.getPosition());
            accountVO.setSex(wUserInfoVO.getSex() == 1 ? "M" : "F");
            accountVO.setHeadPortrait(wUserInfoVO.getHeadimgurl());
            accountVO.setFullName(wUserInfoVO.getNickname());
            accountVO.setTelephone(wUserInfoVO.getTelephone());
            accountVO.setWechat(wechatUserId);

            weChatUserVO.setAccountVO(accountVO);


            if (CollectionUtils.isNotEmpty(orgNameList)) {
                LOGGER.info("获取组织列表返回正常！");
                return weChatUserVO;
            } else {
                LOGGER.info("获取组织列表返回异常!");
                return null;
            }
        } else {
            LOGGER.info("获取组织列表返回异常!");
            return null;
        }
    }

    /**
     * 根据 wechatIds 获取用户微信用信息
     * @param weChatIds
     * @return
     */
    @PostMapping(value = "/get/userInfo/wechatIds")
    public Map<String, UserInfo> getWeChatUserInfoList(@RequestBody List<String> weChatIds) {
        UserInfo userInfo = new UserInfo();
        userInfo.setIsDeleted(Constants.NO_FLAG);
        EntityWrapper<UserInfo> condition = QueryUtil.condition(userInfo);
        condition.in("wechatuserid", weChatIds);
        try {
            List<UserInfo> trAccountUserInfoList = iUserInfoService.selectList(condition);
            if (CollectionUtils.isEmpty(trAccountUserInfoList)) {
                return null;
            }

            Map<String, UserInfo> userInfoMap = trAccountUserInfoList.stream()
                    .collect(Collectors.toMap(a -> a.getWechatuserid(), a -> a));
            return userInfoMap;
        } catch (Exception e) {
            LOGGER.info("异常信息：{}",e.getMessage());
            return null;
        }

    }
}

