package com.yizhi.application.service;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.wechat.application.vo.wechat.SignatureVO;
import com.yizhi.wechat.application.vo.wechat.WUserInfoVO;
import com.yizhi.wechat.application.vo.wechat.WechatUserInfoVO;

/**
 * <p>
 *  服务类
 * </p>
 *
 * @author lilingye
 * @since 2018-04-29
 */
public interface IUserInfoService extends IService<UserInfo> {
    /**
     * 查看微信用户是否在系统中存在
     * @param wechatUserId
     * @return
     */
    TrAccountUserinfo getUserInVmCloud(String wechatUserId);

    /**
     * 用户通过微信登录后 新建关联关系
     * @param wechatUserId
     * @return
     */
    TrAccountUserinfo saveUser(String wechatUserId, Long accountId);

    /**
     *  在微信公众号获取用户信息
     * @param appid
     * @param secret
     * @return
     */
    WechatUserInfoVO getWeChatUserInfo(String appid, String secret, String code, Integer source);

    /**
     *获取企业服务号的用户信息
     * @param publicPlatformConfig 配置实体
     * @return
     */

    WechatUserInfoVO getCompanyWechatUserInfo(PublicPlatformConfig publicPlatformConfig, String code , Integer source);

    /**
     * 获取企业的ticket
     * @param type 接口类型 0 是企业；1 是非企业
     * @param url 当前网页的URL，不包含#及其后面部分
     * @return
     */
    SignatureVO getSignature(String access_token, String url, PublicPlatformConfig publicPlatformConfig);

    /**
     * 获取AccessToken
     * @param publicPlatformConfig APPID
     *
     * @return
     */
    String getAccessToken(PublicPlatformConfig publicPlatformConfig);

    /**
     * 获取jsApiTicket
     * @param access_token ACCESSTOKEN
     * @param publicPlatformConfig 配置信息
     * @return
     */
    String getJsApiTicket(String access_token,PublicPlatformConfig publicPlatformConfig);

    /**
     * 获取企业的用户信息
     * @param userId
     * @param publicPlatformConfig
     * @return
     */
    JSONObject getCwechatUserInfo(String userId,PublicPlatformConfig publicPlatformConfig);

    /**
     * 通过用户openId获取用户信息
     * @param openId
     * @return
     */
    WUserInfoVO getWUserInfoVO(String openId);

    /**
     * 小程序添加用户到表中
     * @param openId
     */
    void saveMiniProgramUser(String openId,String unionId);

    /**
     * 修改用户信息
     * @param userInfo
     * @return flag
     */
    Boolean updateUserInfo(UserInfo userInfo);
}
