package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.feign.CWeChatClient;
import com.yizhi.application.feign.WeChatClient;
import com.yizhi.application.service.IAccessTokenService;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.WTicketVO;
import com.yizhi.wechat.application.vo.wechat.WTokenVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Date;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lilingye
 * @since token
 */
@Service
public class AccessTokenServiceImpl implements IAccessTokenService {

    private static final Logger LOGGER = LoggerFactory.getLogger(AccessTokenServiceImpl.class);

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private CWeChatClient cWeChatClient;
    @Autowired
    private WeChatClient weChatClient;

    @Override
    public String getAccessToken(PublicPlatformConfig publicPlatformConfig) {
        String accessTokenKey = null;
        if (Constants.ZERO.equals(publicPlatformConfig.getType())) {
            accessTokenKey = (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        } else {
            accessTokenKey = publicPlatformConfig.getAppId().trim();
        }
        LOGGER.info("获取当前的accessTokenKey:{}", accessTokenKey);
        //从缓存中获取token
        Object tokenDetail = redisCache.hget(accessTokenKey, accessTokenKey);

        if (null == tokenDetail) {
            // 生成新的token
            return insetWeChatAccessToken(publicPlatformConfig);
        }
        // 判断缓存中的redis存在
        LOGGER.info("当前redis中存放accessToken详细信息：{}", tokenDetail);
        JSONObject accessTokenDetail = JSONObject.parseObject(tokenDetail.toString());
        String accessToken = accessTokenDetail.getString("accessToken");
        if (StringUtils.isEmpty(accessToken) || accessToken.equals(Constants.nullStr)) {
            //生成新的token
            LOGGER.info("accessToken值不在在，重新获取");
            return insetWeChatAccessToken(publicPlatformConfig);
        }

        LOGGER.info("accessToken值存在");
        // 发现从redis中获取的token有过期现象，所以需要检验先=下token的正确性
        JSONObject json = weChatClient.getMenuInfo(accessToken);
        // token 过期
        if (json.toString().contains("40001")) {
            LOGGER.warn("accessToken值存在 过期，重新获取");
            return insetWeChatAccessToken(publicPlatformConfig);
        }
        return accessToken;
    }

    @Override
    public String insetWeChatAccessToken(PublicPlatformConfig publicPlatformConfig) {
        // 调用微信接口获取token 并存在redis中
        String accessToken = null;
        // 根据公众号类型获取
        if ((Constants.ZERO).equals(publicPlatformConfig.getType())) {
            // 获取企业token
            LOGGER.info("企业微信获取token参数：{}", "secret:" + publicPlatformConfig.getSecret().trim() + "--appId:" + publicPlatformConfig.getAppId().trim());
            JSONObject accessTokenJson = cWeChatClient.getWechatToken(publicPlatformConfig.getSecret().trim(), publicPlatformConfig.getAppId().trim());
            LOGGER.info("企业微信accessToken：{}", accessTokenJson);
            accessToken = accessTokenJson.getString(Constants.ACCESS_TOKEN);
        } else {
            // 获取微信号token
            JSONObject accessTokenJson = weChatClient.getAccessToken(publicPlatformConfig.getAppId(), publicPlatformConfig.getSecret());
            LOGGER.info("微信公众号的accessToken：{}", accessTokenJson);
            accessToken = accessTokenJson.getString(Constants.ACCESS_TOKEN);
        }

        String tokenKey = null;

        if ((Constants.ZERO).equals(publicPlatformConfig.getType())) {
            tokenKey = (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        } else {
            tokenKey = publicPlatformConfig.getAppId();
        }
        if (!Constants.nullStr.equals(accessToken) && null != accessToken && !"".equals(accessToken)) {
            WTokenVO wTokenVO = new WTokenVO();
            wTokenVO.setAppid(publicPlatformConfig.getAppId());
            wTokenVO.setCreateTime(new Date());
            wTokenVO.setAccessToken(publicPlatformConfig.getAgentId());
            wTokenVO.setAccessToken(accessToken);
            redisCache.hset(tokenKey, tokenKey, JSON.toJSONString(wTokenVO), 3600);
            return accessToken;
        } else {
            return accessToken;
        }
    }

    /**
     * 定时任务生成jsApiTicket
     *
     * @param publicPlatformConfig
     * @return
     */
    @Override
    public String addJsApiTicket(String accessToken, PublicPlatformConfig publicPlatformConfig) {

        String ticketKey = "jsapiticket_" + (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        LOGGER.info("ticketKey的值是：" + ticketKey);
        String ticket = null;


        WTicketVO wTicketVO = new WTicketVO();
        wTicketVO.setAppid(publicPlatformConfig.getAppId());
        wTicketVO.setAgentId(publicPlatformConfig.getAgentId());
        wTicketVO.setCreateTime(new Date());
        // 企业号的类型
        String type = publicPlatformConfig.getType();
        // 企业服务号
        if (Constants.ZERO.equals(type)) {
            JSONObject jsApiTicketDetail = cWeChatClient.getTicket(accessToken);
            LOGGER.info("生产的jsApiTicket的详情：{}", jsApiTicketDetail);
            ticket = jsApiTicketDetail.getString("ticket");
            LOGGER.info("新生成jsapticket是：" + ticket);
            wTicketVO.setTicket(ticket);
            redisCache.hset(ticketKey, ticketKey, JSON.toJSONString(wTicketVO), Constants.REDIS_INVALID_TIME);
        } else {
            // 微信公众号
            JSONObject jsApiTicketDetail = weChatClient.getTicket(accessToken);
            LOGGER.info("生产的jsApiTicket的详情：{}", jsApiTicketDetail);
            ticket = jsApiTicketDetail.getString("ticket");
            LOGGER.info("重新生成jsapTicket是：" + ticket);
            wTicketVO.setTicket(ticket);
            redisCache.hset(ticketKey, ticketKey, JSON.toJSONString(wTicketVO), Constants.REDIS_INVALID_TIME);
        }
        return ticket;
    }
}
