package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.controller.WechatProviderController;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.IProviderUserInfoService;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.application.task.UpdateUserInfoService;
import com.yizhi.wechat.application.utils.OkHttpUtils;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.WUserInfoVO;
import com.yizhi.wechat.application.vo.wechat.WechatUserInfoVO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 第三方应用suite_ticket 表 服务实现类
 * </p>
 *
 * @author lingye
 * @since 2020-08-05
 */
@Service
@Log4j2
public class ProviderUserInfoServiceImpl implements IProviderUserInfoService {
    @Autowired
    IdGenerator idGenerator;

    @Value("${wechat.provider.user.info}")
    private String providerUserInfoUrl;

    @Value("${wechat.provider.user.detail}")
    private String providerUserDetailUrl;

    @Autowired
    private WechatProviderController providerController;

    @Autowired
    private UpdateUserInfoService updateUserInfoService;

    @Autowired
    private TaskExecutor taskExecutor;

    @Override
    public WechatUserInfoVO getProviderInfoDetail(String suiteId, String code) {

        WechatUserInfoVO userInfoVO = new WechatUserInfoVO();
        String suiteToken = providerController.getSuiteToken(suiteId);
        log.info("suite_token:{}",suiteToken);

        String url = providerUserInfoUrl + "code=" + code + "&access_token=" + suiteToken;
        log.info("当前url：{}",url);
        String result = OkHttpUtils.httpGet(url);
        JSONObject userJson = JSON.parseObject(result);
        log.info("返回的用户信息结果：{}",userJson);


        WUserInfoVO wUserInfoVO = new WUserInfoVO();
        if (!(Constants.ZERO).equals(userJson.getString("errcode"))) {
            userInfoVO.setCode(Constants.DEL_FLAG);
            userInfoVO.setMsg(userJson.getString("errmsg"));
            return userInfoVO;
        }

        String userId = userJson.getString("UserId");
        String openUserId = userJson.getString("open_userid");
        String deviceId = userJson.getString("DeviceId");
        String corpId = userJson.getString("CorpId");


        if (!"null".equals(corpId)) {
            HashMap<String, Object> map = new HashMap<>();
            String userTicket = userJson.getString("user_ticket");
            map.put("user_ticket", userTicket);
            Map<String, String> head = new HashMap<String, String>();
            head.put("Content-Type", "application/json");
            String userDetailUrl = providerUserDetailUrl+"access_token="+suiteToken;
            String userDetail = OkHttpUtils.doRequest2(userDetailUrl, head, JSON.toJSONString(map));
            JSONObject userDetailJSON = JSON.parseObject(userDetail);
            log.info("用户的详情：{}",userDetail);


            if (Constants.ZERO.equals(userDetailJSON.getString("errcode"))) {
                UserInfo userInfo = new UserInfo();

                getProviderUser(userDetailJSON, userInfo);
                userInfo.setDeviceId(deviceId);
                userInfo.setUserId(userId);
                BeanUtils.copyProperties(userInfo, wUserInfoVO);
                updateUserInfoService.taskAddProviderUserInfo(userInfo);
            }
            log.info("返回的实体wUserInfoVO:{}", JSON.toJSON(wUserInfoVO));

            wUserInfoVO.setWechatuserid(openUserId);
            wUserInfoVO.setIsDeleted(Constants.NO_FLAG);

            userInfoVO.setWUserInfoVO(wUserInfoVO);
            userInfoVO.setCode(Constants.YES_FLAG);
        }
        return userInfoVO;
    }


    public void getProviderUser(JSONObject userDetailJSON, UserInfo userInfo) {

        userInfo.setId(idGenerator.generate());
        userInfo.setWechatuserid(userDetailJSON.getString("open_userid"));
        userInfo.setNickname(userDetailJSON.getString("name"));
        userInfo.setMobile(userDetailJSON.getString("mobile"));
        userInfo.setEmail(userDetailJSON.getString("email"));
        userInfo.setHeadimgurl(userDetailJSON.getString("avatar"));
        userInfo.setQrCode(userDetailJSON.getString("qr_code"));
        userInfo.setSex(Integer.parseInt(userDetailJSON.getString("gender")));
        String corpId = userDetailJSON.getString("corpid");
        userInfo.setCorpId(corpId);
        userInfo.setCreateTime(new Date());
        userInfo.setUserType(Constants.DEL_FLAG);

    }

}
