package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.application.controller.WechatProviderController;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.feign.CWeChatClient;
import com.yizhi.application.feign.WeChatClient;
import com.yizhi.application.mapper.TrAccountUserinfoMapper;
import com.yizhi.application.mapper.UserInfoMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.IUserInfoService;
import com.yizhi.application.task.UpdateUserInfoService;
import com.yizhi.wechat.application.vo.wechat.*;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

//import com.yizhi.core.application.task.InsertCompanyUserInfoService;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lilingye
 * @since 2018-04-29
 */
@Service
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements IUserInfoService {

    private static final Logger LOGGER = LoggerFactory.getLogger(UserInfoServiceImpl.class);
    @Resource
    private TrAccountUserinfoMapper trAccountUserinfoMapper;

    @Resource
    private UserInfoMapper userInfoMapper;

    @Resource
    private IdGenerator idGenerator;

    @Autowired
    private CWeChatClient cWeChatClient;

    @Autowired
    private WeChatClient weChatClient;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IUserInfoService iUserInfoService;
    @Autowired
    private UpdateUserInfoService updateUserInfoService;

    @Autowired
    private WechatProviderController wechatProviderController;

    /**
     * 查询用户是wmcloud 系统
     *
     * @param wechatUserId
     * @return
     */
    @Override
    public TrAccountUserinfo getUserInVmCloud(String wechatUserId) {
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setWechatuserid(wechatUserId);
        trAccountUserinfo.setIsDeleted(Constants.DEL_FLAG);
        return trAccountUserinfoMapper.selectOne(trAccountUserinfo);
    }

    @Override
    public TrAccountUserinfo saveUser(String wechatUserId, Long accountId) {
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setId(idGenerator.generate());
        trAccountUserinfo.setAccountId(accountId);
        trAccountUserinfo.setWechatuserid(wechatUserId);
        trAccountUserinfoMapper.insert(trAccountUserinfo);
        return trAccountUserinfoMapper.selectOne(trAccountUserinfo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public WechatUserInfoVO getWeChatUserInfo(String appId, String secret, String code, Integer source) {
        WechatUserInfoVO userInfoVO = new WechatUserInfoVO();
        String weChatUserId = null;
        String unionId = null ;
        try {
            JSONObject tokenJSON = weChatClient.getWechaToken(appId, secret, code);
            LOGGER.info("用户信息的token：{}", tokenJSON);
            if (tokenJSON.getString("errcode") != null && !(Constants.nullStr).equals(tokenJSON.getString("errcode"))) {
                // 接口报错返回信息
                userInfoVO.setCode(Constants.NO_FLAG);
                userInfoVO.setMsg(tokenJSON.getString("errmsg"));
                return userInfoVO;
            }
            String accessToken = tokenJSON.getString("access_token");
            LOGGER.info("获取微信用户新的token：{}", accessToken);
            weChatUserId = tokenJSON.getString("openid");
            unionId = tokenJSON.getString("unionid");
            LOGGER.info("用户的openId：{}", weChatUserId);

            WUserInfoVO wUserInfoVO = new WUserInfoVO();
            if (source == null) {
                updateUserInfoService.taskAddUserInfo(accessToken, weChatUserId,unionId);
            } else if (source == 1){
                UserInfo userInfo = getWeChatUserInfo(accessToken, weChatUserId);
                BeanUtils.copyProperties(userInfo,wUserInfoVO);
            }
            wUserInfoVO.setWechatuserid(weChatUserId);
            userInfoVO.setWUserInfoVO(wUserInfoVO);
            userInfoVO.setCode(Constants.YES_FLAG);
        } catch (Exception e) {
            LOGGER.info("用户异常信息：{}", e.getMessage());
        }
        return userInfoVO;
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public WechatUserInfoVO getCompanyWechatUserInfo(PublicPlatformConfig publicPlatformConfig, String code, Integer source) {
        WechatUserInfoVO userInfoVO = new WechatUserInfoVO();
        publicPlatformConfig.setType("0");

        String redisKey = (publicPlatformConfig.getAppId().trim() + publicPlatformConfig.getAgentId()).trim();
        //  声明 access_token
        String accessToken = getCwechatAccessToken(redisKey, publicPlatformConfig);

        LOGGER.info("access_token is：" + accessToken);

        JSONObject userInfoJSON = cWeChatClient.getUserInfo(accessToken, code);

        LOGGER.info("userInfoJSON：" + userInfoJSON);
        String userTicket = userInfoJSON.getString("user_ticket");
        LOGGER.info("企业的user-ticket是：" + userTicket);

        WechatParamVO wechatParamVO = new WechatParamVO();
        wechatParamVO.setUser_ticket(userTicket);
        JSONObject userDetailJSON = null;
        String userId = null;
        try {
            userDetailJSON = cWeChatClient.getUserDetail(accessToken, wechatParamVO);
            if (!(Constants.ZERO).equals(userDetailJSON.getString("errcode"))) {
                userInfoVO.setCode(Constants.DEL_FLAG);
                userInfoVO.setMsg(userDetailJSON.getString("errmsg"));
                return userInfoVO;
            }
            userId = userDetailJSON.getString("userid");
        } catch (Exception e) {
            LOGGER.info("" + e.getLocalizedMessage());
        }
        WUserInfoVO wUserInfoVO = new WUserInfoVO();

        if(source==null) {
            updateUserInfoService.taskAddCompanyUserInfo(userDetailJSON);
        } else if (source == 1){
            UserInfo userInfo = getCompanyUser(userDetailJSON);
            BeanUtils.copyProperties(userInfo,wUserInfoVO);
        }
        wUserInfoVO.setWechatuserid(userId);
        wUserInfoVO.setIsDeleted(Constants.NO_FLAG);

        userInfoVO.setWUserInfoVO(wUserInfoVO);
        userInfoVO.setCode(Constants.YES_FLAG);

        return userInfoVO;

    }

    @Override
    public SignatureVO getSignature(String accessToken, String url, PublicPlatformConfig publicPlatformConfig) {

        String ticketKey = "jsapiticket_" + (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        String ticket = null;
        //从缓存中获取token
        Object ticketVO = redisCache.hget(ticketKey, ticketKey);
        // 0 微信企业服务请求
        if (ticketVO != null) {
            Object ticketJSON = JSON.toJSON(ticketVO);
            LOGGER.info("查看redis中ticket信息：" + ticketJSON.toString());
            ticket = JSON.parseObject(ticketJSON.toString()).getString("ticket");
            if (!"".equals(ticket) && !(Constants.nullStr).equals(ticket) && ticket != null) {
                LOGGER.info("redis中ticket信息：" + ticket);
            } else {
                //生成新的jsApiTicket
                ticket = getJsApiTicket(accessToken, publicPlatformConfig);
                LOGGER.info("新生成的ticket:" + ticket);
            }
        } else {

            ticket = getJsApiTicket(accessToken, publicPlatformConfig);
            LOGGER.info("新生成的ticket:" + ticket);
        }
        return sign(ticket, url);
    }

    @Override
    public String getAccessToken(PublicPlatformConfig publicPlatformConfig) {
        LOGGER.info("获取微信token的参数： " + publicPlatformConfig);
        String accessToken = null;
        // 根据公众号类型获取
        if ((Constants.ZERO).equals(publicPlatformConfig.getType())) {
            // 获取企业token
            JSONObject accessTokenJson = cWeChatClient.getWechatToken(publicPlatformConfig.getSecret().trim(), publicPlatformConfig.getAppId().trim());
            LOGGER.info("调用企业微信接返回的信息：" + accessTokenJson);
            accessToken = accessTokenJson.getString(Constants.ACCESS_TOKEN);
            LOGGER.info("调用企业微信接口生成的AccessToken：" + accessToken);
        } else if ((Constants.ONE).equals(publicPlatformConfig.getType()))  {
            // 获取微信号token
            JSONObject accessTokenJson = weChatClient.getAccessToken(publicPlatformConfig.getAppId(), publicPlatformConfig.getSecret());
            LOGGER.info("appid是" + publicPlatformConfig.getAppId());
            LOGGER.info("secret是" + publicPlatformConfig.getSecret());
            LOGGER.info("accessTokenJson的值" + accessTokenJson.toJSONString());
            accessToken = accessTokenJson.getString(Constants.ACCESS_TOKEN);

        }else if ((Constants.TWO).equals(publicPlatformConfig.getType())){
            accessToken = wechatProviderController.getSuiteToken(publicPlatformConfig.getAppId().trim());
        }
            return accessToken;
    }

    @Override
    public JSONObject getCwechatUserInfo(String userId, PublicPlatformConfig publicPlatformConfig) {

        String redisKey = (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        LOGGER.info("accessToken 的key是：" + redisKey);
        String accessToken = getCwechatAccessToken(redisKey, publicPlatformConfig);
        LOGGER.info("accessToken：" + accessToken);
        JSONObject cuserInfo = cWeChatClient.getCWechatUserDetail(accessToken, userId);

        LOGGER.info("获取到的企业用户的信息：" + cuserInfo.toJSONString());
        return cuserInfo;
    }

    @Override
    public WUserInfoVO getWUserInfoVO(String openId) {
        LOGGER.info("传来的值openId：" + openId);
        UserInfo wechatUserInfo = new UserInfo();

        wechatUserInfo.setWechatuserid(openId);
        wechatUserInfo.setIsDeleted(Constants.DEL_FLAG);
        WUserInfoVO userInfoVO = new WUserInfoVO();
        UserInfo userInfo = null;
        List<UserInfo> userInfoList = userInfoMapper.selectList(QueryUtil.condition(wechatUserInfo));

        if (CollectionUtils.isNotEmpty(userInfoList)) {
            userInfo = userInfoList.get(0);
        }
        LOGGER.info("查询用户openId的数据{}", userInfo);
        if (userInfo != null) {
            BeanUtils.copyProperties(userInfo, userInfoVO);
            return userInfoVO;
        } else {
            return null;
        }
    }


    /**
     * 获取签名
     *
     * @param jsApiTicket
     * @param url
     * @return
     */

    public static SignatureVO sign(String jsApiTicket, String url) {

        Map<String, String> ret = new HashMap<String, String>();
        String nonceStr = createNonceStr();
        String timestamp = createTimeStamp();
        String signature = "";
        StringBuffer stringBuffer = new StringBuffer();
        //注意这里参数名必须全部小写，且必须有序

        stringBuffer.append("jsapi_ticket=").
                append(jsApiTicket).
                append("&noncestr=").
                append(nonceStr).append("&timestamp=").append(timestamp).append("&url=").append(url);

        LOGGER.info("拼接字符串：" + stringBuffer.toString());

        try {
            MessageDigest crypt = MessageDigest.getInstance("SHA-1");
            crypt.reset();
            crypt.update(stringBuffer.toString().getBytes("UTF-8"));
            signature = byteToHex(crypt.digest());
            LOGGER.info("签名后的值：" + signature);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        SignatureVO signatureVO = new SignatureVO();
        signatureVO.setSignnatrue(signature);
        signatureVO.setTimestamp(timestamp);
        signatureVO.setNoncestr(nonceStr);

        return signatureVO;
    }

    private static String byteToHex(final byte[] hash) {
        Formatter formatter = new Formatter();
        for (byte b : hash) {
            formatter.format("%02x", b);
        }
        String result = formatter.toString();
        formatter.close();
        return result;
    }

    /**
     * 获取随机数
     *
     * @return
     */
    private static String createNonceStr() {
        return UUID.randomUUID().toString();
    }

    /**
     * 获取时间戳
     *
     * @return
     */
    private static String createTimeStamp() {
        return Long.toString(System.currentTimeMillis() / 1000);
    }

    /**
     * 获取企业服务号的accessToken
     *
     * @param redisKey
     * @param publicPlatformConfig
     * @return
     */
    public String getCwechatAccessToken(String redisKey, PublicPlatformConfig publicPlatformConfig) {

        //从缓存中获取token
        Object tokenVO = redisCache.hget(redisKey, redisKey);
        LOGGER.info("从缓存中获取的token：" + tokenVO);

        // 声明 access_token
        String access_token = null;

        if (!org.springframework.util.StringUtils.isEmpty(tokenVO)) {
            // 获取token
            Object tokenJSON = JSON.toJSON(tokenVO);
            LOGGER.info("查看缓存中的access_token信息：" + tokenJSON.toString());
            LOGGER.info("缓存中的accessToken的值：" + JSON.parseObject(tokenJSON.toString()).getString("accessToken"));
            String accessToken = JSON.parseObject(tokenJSON.toString()).getString("accessToken");

            if (!"".equals(accessToken) && !(Constants.nullStr).equals(accessToken) && accessToken != null) {
                LOGGER.info("accessToken值存在");
                access_token = JSON.parseObject(tokenJSON.toString()).getString("accessToken");
            } else {
                // 生成新的token
                String accToken = getAccessToken(publicPlatformConfig);
                Date nowDate = new Date();
                WTokenVO wTokenVO = new WTokenVO();
                wTokenVO.setAppid(publicPlatformConfig.getAppId());
                wTokenVO.setCreateTime(nowDate);
                wTokenVO.setAccessToken(access_token);
                redisCache.hset(redisKey, redisKey, accToken, Constants.REDIS_INVALID_TIME);
                access_token = getAccessToken(publicPlatformConfig);
            }
        } else {
            WTokenVO wTokenVO = new WTokenVO();
            wTokenVO.setAppid(publicPlatformConfig.getAppId());
            wTokenVO.setCreateTime(new Date());

            // 调用微信接口获取token 并存在redis中
            access_token = iUserInfoService.getAccessToken(publicPlatformConfig);
            wTokenVO.setAccessToken(access_token);
            redisCache.hset(redisKey, redisKey, JSON.toJSONString(wTokenVO), Constants.REDIS_INVALID_TIME);
        }
        LOGGER.info("accessToken的值是：" + access_token);
        return access_token;

    }

    /**
     * 生成签名
     *
     * @param publicPlatformConfig
     * @return
     */
    @Override
    public String getJsApiTicket(String accessToken, PublicPlatformConfig publicPlatformConfig) {

        String ticketKey = "jsapiticket_" + (publicPlatformConfig.getAppId() + publicPlatformConfig.getAgentId()).trim();
        LOGGER.info("ticketKey的值是：" + ticketKey);
        String ticket = null;
        // 企业号的类型
        String type = publicPlatformConfig.getType();

        WTicketVO wTicketVO = new WTicketVO();
        wTicketVO.setAppid(publicPlatformConfig.getAppId());
        wTicketVO.setAgentId(publicPlatformConfig.getAgentId());
        wTicketVO.setCreateTime(new Date());

        // 企业服务号
        if (Constants.ONE.equals(type)) {
            // 微信公众号
            JSONObject jsApiTicketJSON = weChatClient.getTicket(accessToken);
            LOGGER.info("=====jsApiTicketJSON========" + jsApiTicketJSON.toJSONString());
            ticket = jsApiTicketJSON.getString("ticket");
            LOGGER.info("重新生成jsapTicket是：" + ticket);
            wTicketVO.setTicket(ticket);
            redisCache.hset(ticketKey, ticketKey, JSON.toJSONString(wTicketVO), Constants.REDIS_INVALID_TIME);
        } else {
                JSONObject jsApiTicketJSON = cWeChatClient.getTicket(accessToken);
                LOGGER.info(jsApiTicketJSON.toJSONString());
                ticket = jsApiTicketJSON.getString("ticket");
                LOGGER.info("新生成jsapTicket是：" + ticket);
                wTicketVO.setTicket(ticket);
                redisCache.hset(ticketKey, ticketKey, JSON.toJSONString(wTicketVO), Constants.REDIS_INVALID_TIME);
        }

        return ticket;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveMiniProgramUser(String openId,String unionId) {
        UserInfo wechatUserInfo = new UserInfo();
        wechatUserInfo.setWechatuserid(openId);
        wechatUserInfo.setUnionId(unionId);
        wechatUserInfo.setIsDeleted(Constants.DEL_FLAG);
        List<UserInfo> userInfos = this.selectList(QueryUtil.condition(wechatUserInfo));

        if (CollectionUtils.isEmpty(userInfos)) {
            UserInfo userInfo = new UserInfo();
            userInfo.setId(idGenerator.generate());
            userInfo.setWechatuserid(openId);
            userInfo.setUnionId(unionId);
            // 小程序类型
            userInfo.setUserType(4);
            userInfo.setCreateTime(new Date());
            userInfo.setUserType(Constants.DEL_FLAG);
            this.insert(userInfo);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateUserInfo(UserInfo userInfo) {

        return this.baseMapper.updateUserInfoByWeChatId(userInfo.getNickname(),userInfo.getHeadimgurl(),userInfo.getWechatuserid());
    }

    /**
     * 获取微信用户的信息
     * @param accessToken
     * @param weChatUserId
     * @return
     */
    public UserInfo getWeChatUserInfo(String accessToken,String weChatUserId){
        JSONObject user = weChatClient.getUserInfo(accessToken, weChatUserId);
        UserInfo userInfo = new UserInfo();
        userInfo.setId(idGenerator.generate());
        userInfo.setWechatuserid(user.getString("openid") != null ? user.getString("openid") : null);
        userInfo.setNickname(user.getString("nickname") != null ? user.getString("nickname") : null);
        userInfo.setProvince(user.getString("province") != null ? user.getString("province") : null);
        userInfo.setCity(user.getString("city") != null ? user.getString("city") : null);
        userInfo.setHeadimgurl(user.getString("headimgurl") != null ? user.getString("headimgurl") : null);
        userInfo.setPrivilege(user.getString("privilege") != null ? user.getString("privilege") : null);
        userInfo.setUnionId(user.getString("unionid") != null ? user.getString("unionid") : null);
        userInfo.setCreateTime(new Date());
        userInfo.setUserType(Constants.YES_FLAG);
        return userInfo;
    }


    public UserInfo getCompanyUser(JSONObject userDetailJON) {
        UserInfo userInfo = new UserInfo();
        userInfo.setId(idGenerator.generate());
        userInfo.setWechatuserid(userDetailJON.getString("userid"));
        userInfo.setNickname(userDetailJON.getString("name"));
        userInfo.setMobile(userDetailJON.getString("mobile"));
        userInfo.setEmail(userDetailJON.getString("email"));
        userInfo.setHeadimgurl(userDetailJON.getString("avatar"));
        userInfo.setQrCode(userDetailJON.getString("qr_code"));
        userInfo.setCreateTime(new Date());
        userInfo.setUserType(Constants.DEL_FLAG);

       return userInfo;
    }

}
